-- Do not use this file except in compliance with the License. You may
-- obtain a copy of the License at http://www.microcore.org/License/
-- Software distributed under the License is distributed on an "AS IS" basis,
-- WITHOUT WARRANTY OF ANY KIND, either express or implied.
-- See the License for the specific language governing rights and limitations
-- under the License.
--
-- The Original Code is: MicroCore ROM_BENCH.VHD 1.10
--
-- The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
--
-- Introduction  see: information/uCore_Introduction.pdf
-- Tools         see: information/uCore_Implementation.pdf
-- Versions      see: uCore_Versions.txt
-- License       see: information/uCore_License.pdf
-- Conventions   see: information/uCore_Conventions.pdf
-- Contributors  see: information/uCore_Contributors.pdf
-- Philosophy    see: information/uCore_Philosophy.pdf
--
-- Test bench to be used with internal asynchronous data RAM and external 
-- program ROM.

LIBRARY IEEE;
USE     IEEE.std_logic_1164.ALL;
USE     work.functions.ALL;
USE     work.constants.ALL;
                                
ENTITY bench IS
END bench;

ARCHITECTURE testbench OF bench IS

COMPONENT core
PORT (reset       : IN    std_logic;
      clk         : IN    std_logic;
      clk_en      : IN    std_logic;
      delay       : INOUT std_logic; -- unused pad, serves to generate tri_en signal
-- io interface
      iosel       : OUT   std_logic;
      iotrap      : IN    std_logic;
      iowrite     : OUT   std_logic;
      ioaddr      : OUT   data_addr;
      iodout      : OUT   data_bus;
      iodin       : IN    data_bus;
       
      romaddr     : OUT   prog_addr;
      instruction : IN    inst_bus;
-- Data memory
--      ramwr       : OUT   std_logic;
--      ramaddr     : OUT   data_addr;
--      ramdin      : OUT   data_bus;
--      ramdout     : IN    data_bus;
      int_in      : IN    int_bus);
END COMPONENT;

COMPONENT data_ram
PORT (dram : INOUT dram_port);
END COMPONENT;

COMPONENT code_rom
PORT (rom : INOUT rom_port);
END COMPONENT;

SIGNAL reset      : std_logic; 
SIGNAL clk        : std_logic;
SIGNAL clk_en     : std_logic; 
SIGNAL iotrap     : std_logic;  
SIGNAL interrupt  : int_bus;

SIGNAL iosel      : std_logic;
SIGNAL iowrite    : std_logic;
SIGNAL ioaddr     : data_addr;
SIGNAL iodout     : data_bus;
SIGNAL iodin      : data_bus;
SIGNAL ioreg      : data_bus; 

SIGNAL rom        : rom_port; 
--SIGNAL dram       : dram_port; 

CONSTANT IOPORT1  : data_addr := to_vec(-5, data_addr_width); -- for testbench
CONSTANT IOPORT2  : data_addr := to_vec(-6, data_addr_width); -- for testbench

CONSTANT cycle    : TIME := 100 ns;

BEGIN

fpgacore: core PORT 
MAP ( reset   => reset,
      clk     => clk,
      clk_en  => clk_en,

      iosel   => iosel,
      iotrap  => iotrap,
      iowrite => iowrite,
      ioaddr  => ioaddr,
      iodout  => iodout,
      iodin   => iodin,

      romaddr => rom.addr,
      instruction    => rom.data,

--      ramwr   => dram.write,
--      ramaddr => dram.addr,
--      ramdin  => dram.din,
--      ramdout => dram.dout,

      int_in  => interrupt);

program: code_rom PORT MAP(rom => rom);

--data_memory: data_ram PORT MAP (dram => dram);
--dram.clk <= clk;

reset_core: PROCESS
BEGIN
   reset        <= '1';
   clk_en       <= '1';
   interrupt    <= "00";
   iotrap       <= '0';
   WAIT FOR cycle*2 + 20 ns;
   reset        <= '0';
   WAIT;
END PROCESS reset_core;

testio: PROCESS(reset, clk)
BEGIN
  IF  reset='1'  THEN
     ioreg <= (OTHERS => '0');
  ELSIF  rising_edge(clk)  THEN
     IF  clk_en='1'  THEN
       IF  iosel='1' AND iowrite='1' AND ioaddr=IOPORT1  THEN
           ioreg <= iodout;
       END IF;
     END IF;
  END IF;
END PROCESS testio;

iodin <= ioreg WHEN  ioaddr=IOPORT1  ELSE  to_vec(85, data_width);

clock: PROCESS
BEGIN
  LOOP
    clk <= '1';
    WAIT FOR cycle/2;
    clk <= '0';
    WAIT FOR cycle/2;
  END LOOP;
END PROCESS clock;

END testbench;