-- Do not use this file except in compliance with the License. You may
-- obtain a copy of the License at http://www.microcore.org/License/
-- Software distributed under the License is distributed on an "AS IS" basis,
-- WITHOUT WARRANTY OF ANY KIND, either express or implied.
-- See the License for the specific language governing rights and limitations
-- under the License.
--
-- The Original Code is: MicroCore SYN_DATA_RAM.VHD 1.10
--
-- The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
--
-- Introduction  see: information/uCore_Introduction.pdf
-- Tools         see: information/uCore_Implementation.pdf
-- Versions      see: uCore_Versions.txt
-- License       see: information/uCore_License.pdf
-- Conventions   see: information/uCore_Conventions.pdf
-- Contributors  see: information/uCore_Contributors.pdf
-- Philosophy    see: information/uCore_Philosophy.pdf
--
-- The data/return stack RAM implemented in such a way
-- that it should be recognised as synchronous 
-- RAM blocks by the synthesiser.

LIBRARY IEEE;
USE     IEEE.std_logic_1164.ALL;
USE     work.functions.ALL;
USE     work.constants.ALL;

-- The whole RAM interface is contained in RECORD dram_port
ENTITY data_ram IS
PORT (dram : INOUT dram_port);
END data_ram;

ARCHITECTURE synchronous_read_ram OF data_ram IS

TYPE ram_data IS ARRAY (INTEGER RANGE 0 TO 2**data_addr_width-1) OF data_bus;

SIGNAL ram    : ram_data;
SIGNAL clk    : std_logic;
SIGNAL addr   : data_addr;

BEGIN

clk  <= dram.clk;  -- for the synthesizer

synch_ram: PROCESS(clk)
BEGIN
  IF  rising_edge(clk)  THEN
     addr <= dram.addr;
     IF  dram.write='1'  THEN
         ram(to_int('0' & dram.addr)) <= dram.din;
     END IF;
  END IF;
END PROCESS synch_ram;

dram.dout <= ram(to_int('0' & addr));

END synchronous_read_ram;
