\ Do not use this file except in compliance with the License. You may
\ obtain a copy of the License at http://www.microcore.org/License/
\ Software distributed under the License is distributed on an "AS IS" basis,
\ WITHOUT WARRANTY OF ANY KIND, either express or implied.
\ See the License for the specific language governing rights and limitations
\ under the License.
\
\ The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
\
\ Introduction  see: information/uCore_Introduction.pdf
\ Tools         see: information/uCore_Implementation.pdf
\ Versions      see: uCross_Versions.txt
\ License       see: information/uCore_License.pdf
\ Conventions   see: information/uCore_Conventions.pdf
\ Contributors  see: information/uCore_Contributors.pdf
\ Philosophy    see: information/uCore_Philosophy.pdf
\
\ MicroCore monitor for the debugger interface
\ hardware must be configured with an appropriate bidirectional debug interface
\
forget Image  Create Image     \ prune dicitionary and compile IMAGE again, so target cide can be loaded several times on top of cross-compiler
New                            \ reset target compiler
Target                         \ switch into target compilation mode and context

include constants.f            \ MicroCore Register addresses and bits

#pause 8 + code-origin
         0 data-origin

 2 Constant #pcint

: set_carry    ( -- )      status@ #c  or status! ;
: reset_carry  ( -- )      status@ [ #c invert ] Literal and status! ;
: smallest     ( -- min )  0 rol -2 or u2/ invert ;                      \ carry bit unaffected
: 0<          ( n1 -- f )  dup sign IF  invert  THEN  xor ;
: d+      ( d1 d2 -- d3 )  >r swap >r +  r> r> +c ;
: d-      ( d1 d2 -- d3 )  >r swap >r -  r> r> -c ;

: in?     ( -- f )  flags@ #pcint and ;

: in@     ( -- w )  BEGIN  in? UNTIL  DEBUG_REG @ ;

: out!    ( w -- )  DEBUG_REG ! ;

: execute  ( addr -- )  >r ;

: fill ( addr len pattern -- )
   -rot ?dup IF  swap 1- swap  1- FOR  over swap 1 ++!  NEXT  THEN  drop drop
;
: clear  ( ??? -- )
   BEGIN  status@ #dsempty and 0= WHILE  drop  REPEAT
   r>  BEGIN  status@ #rsempty and 0= WHILE  r> drop  REPEAT  >r
;
: asmopcodes ( ??? -- ??? )   \ do not change this definition
  Label \drop  drop      EXIT
  Label \swap  swap      EXIT
  Label \over  over      EXIT
  Label \dup   dup       EXIT
  Label \st    !         EXIT
  Label \ld    @         EXIT
  Label \add   +         EXIT
  Label \sub   -         EXIT
  Label \not   invert    EXIT
  Label \or    or        EXIT
  Label \and   and       EXIT
  Label \xor   xor       EXIT
  Label \sl    2*        EXIT
  Label \lsr   u2/       EXIT
  Label \ror   ror       EXIT
  Label \rol   rol       EXIT
  Label \zequ  0=        EXIT
;
: receivestack ( -- ??? )
   in@ ?dup 0=EXIT       \ number of elements
   1- FOR in@ NEXT
;
: sendstack    ( ??? -- )
   BEGIN  status@ #dsempty and 0= WHILE  -1 out! out!  REPEAT
\   BEGIN  status@ #rsempty and 0= WHILE   1 out! r> dup out!  REPEAT     \ include for return-stack
\   BEGIN  status@ #dsempty and 0= WHILE  >r  REPEAT                      \ display during debugging
;
: monitor  ( -- )
   in? IF  in@ drop  THEN   0 out!
   BEGIN  receivestack in@ execute sendstack  REPEAT
;
: cold  ( xx -- )  clear  ['] monitor nop BRANCH 
;
: debugger ( -- )  \ main debugger loop called via pause
   smallest out!   ['] monitor nop BRANCH
;
: msg      ( n -- )    \ utilities
    dup out!           \ transfer message to host
    0< ?EXIT           \ if negative (error causing abort in host)
    cold               \ then restart debug loop
;
: comminit ( -- )             \ initialisation of TARGET <-> HOST communication
   BEGIN  $A0A out!   in@ $5f5 xor 0= UNTIL  $505 out!  in@ drop
;
: init     ( -- )   comminit cold ;

: nextstep ( -- ) r> drop  in@ >r ;  \ call and jump (modified via >r)
: gettor   ( -- ) r> r>   out! >r ;  \ get tor and transfer to host
: puttor   ( -- ) r> in@    >r >r ;  \ transfer from host and put tor

: ei       ( -- )    \ enable all interrupts
   status@ #ie or status!   -1 ie!              
;
: IntServer ( -- )  DEBUG_REG @ drop
IRET ;

: test  ( -- )   1 2 3 0    ei
   smallest 7 FOR  1+ dup DEBUG_REG !  NEXT  drop
   + - +      IF  $100 BEGIN REPEAT THEN
   0 BEGIN REPEAT
;

\ here #reset code-origin ] ['] init      nop BRANCH [ code-origin     \ compile BRANCH to INIT at reset vector location
here #reset code-origin ] ['] test      nop BRANCH [ code-origin     \ compile BRANCH to INIT at reset vector location
here #irq   code-origin ] ['] IntServer nop BRANCH [ code-origin
here #pause code-origin ] r> 1- >r            IRET [ code-origin 

Host                                 \ back into the Host environment

